/* libStreamedXML/src/lib/Core/Exceptions.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lsx {



Exception::Exception(const std::wstring& reason, int line, int col)
    : reason(reason), line(line), col(col)
{
}



const char* Exception::what() const throw()
{
    if(utf8Reason.empty()) utf8Reason = utf8::encode(reason, true);
    return utf8Reason.c_str();
}



NotWellFormed::NotWellFormed(const std::wstring& error, int line, int col)
    : Exception(format(error, line, col), line, col), error(error)
{
}



std::wstring NotWellFormed::format(const std::wstring& error, int line, int col)
{
    std::wostringstream ost;
    ost << L"Streamed XML is not well formed:"
        << L"\n  line  : " << line + 1
        << L"\n  column: " << col + 1
        << L"\n  reason: " << error;
    return ost.str();
}



UnknownEntity::UnknownEntity(const std::wstring& name, int line, int col)
    : Exception(format(name, line, col), line, col), name(name)
{
}



std::wstring UnknownEntity::format(const std::wstring& name, int line, int col)
{
    std::wostringstream ost;
    ost << L"Encountered an unknown entity in the Streamed XML:"
        << L"\n  line  : " << line + 1
        << L"\n  column: " << col + 1
        << L"\n  name  : " << name;
    return ost.str();
}



}
/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
