/* libStreamedXML/src/tests/Callback.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

#include "StreamedXML"
#include <iostream>
#include <fstream>



class PrintCallback : public lsx::Callback {
public:
    virtual void comment(const std::wstring& data)
    {
        std::wcout << L"Callback: comment: ``" << data << L"''.\n";
    }

    virtual void PI(const std::wstring& target, const std::wstring& data)
    {
        std::wcout << L"Callback: PI: ``" << target << L"'': ``"
            << data << L"''.\n";
    }

    virtual void element(const std::wstring& elemName,
                         const std::map<std::wstring, std::wstring>& attrs)
    {
        std::wcout << L"Callback: element: ``" << elemName << "''\n";
        for(std::map<std::wstring, std::wstring>::const_iterator i = attrs.begin(), 
            end = attrs.end(); i != end; ++i)
        {
            std::wcout << L"Callback:          attribute: "
                << i->first << L"=``" << i->second << L"''\n";
        }
        std::wcout << L"Callback:          end of attributes.\n";
    }

    virtual void closeTag(const std::wstring& elemName)
    {
        std::wcout << L"Callback: close element: ``" << elemName << "''.\n";
    }

    virtual void whiteSpace(const std::wstring& space)
    {
        (void)space;
        std::wcout << L"Callback: whitespace.\n";
    }

    virtual void content(const std::wstring& data)
    {
        std::wcout << L"Callback: content: ``" << data << L"''.\n";
    }

    virtual void cdata(const std::wstring& data)
    {
        std::wcout << L"Callback: CDATA: ``" << data << "''.\n";
    }

    virtual std::wstring entityRef(const std::wstring& name)
    {
        std::wcout << L"Callback: entity: ``" << name
            << L"'' (returning as data).\n";
        return name;
    }
};



//END // Parser class //////////////////////////////////////////////////



int main(int argc, char* argv[])
{
    if(argc == 2 && !strcmp(argv[1], "--print-summary")) {
        std::wcout << L"Tests the callback functions.\n";
        return 0;
    }

    if(argc != 2) {
        std::wcerr << L"Expecting name of XML file.\n";
        return 1;
    }

    std::ifstream fin(argv[1]);
    if(!fin) {
        std::wcerr << L"Couldn't open file for input.\n";
        return 1;
    }

    int ret = 0;
    try {
        PrintCallback printCallback;
        lsx::Parser xmlParser(&printCallback, true);
        lsx::Decoder xmlDecoder(&xmlParser);
        char data[1024];
        
        while(!fin.eof()) {
            fin.read(data, sizeof(data));
            xmlDecoder.feedData(data, fin.gcount());
        }
    }
    catch(std::exception& e) {
        std::cerr << e.what();
        ret = 1;
    }

    return ret;
}
