/* libStreamedXML/src/tests/UTF-8.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

#include "StreamedXML"
#include <iostream>
#include <utf8>



class TestFailed { };



class MySimpleParser : private lsx::Callback {
private:
    lsx::Parser xmlParser;
    lsx::Decoder xmlDecoder;

public:
    MySimpleParser()
    : xmlParser(this, true), xmlDecoder(&xmlParser)
    {
    }

    void feedBadData(const char* bytes, size_t amount)
    {
        try {
            xmlDecoder.feedData(bytes, amount);
        }
        catch(utf8::Error&) {
            return;
        }

        throw TestFailed();
    }

    virtual void feedData(const char* bytes, size_t amount)
    {
        try {
            xmlDecoder.feedData(bytes, amount);
        }
        catch(std::exception& e) {
            std::wcerr << e.what() << std::endl;
            throw TestFailed();
        }
    }
};



char restartMarker[] = "<![RESTART[[]]>";
char okData[] = "<element>"
                "\x41\xE2\x89\xA2\xCE\x91\x2E"
                "\xED\x95\x9C\xEA\xB5\xAD\xEC\x96\xB4"
                "\xE6\x97\xA5\xE6\x9C\xAC\xE8\xAA\x9E";

char badDataOverlong[] = "<overlong>"
                         "\xC0\xB0"; // ascii '0', overlong
char badDataMalformed0[] = "<malformed>"
                           "\xFF"; // never appears
char badDataMalformed2[] = "<malformed>"
                           "\x8F"; // lone continuation char
char badDataMalformed3[] = "<malformed>"
                           "\xC1\x3F"; // missing continuation char

struct TestData {
    wchar_t* desc;
    char* str;
    size_t len;
};

#define P(D, Q) { D, Q, sizeof( Q ) - 1 }
TestData testData[] = {
    P(L"Overlong character encoding", badDataOverlong),
    P(L"Illegal octet (0xFF)", badDataMalformed0),
    P(L"Lone continuation char", badDataMalformed2),
    P(L"Missing continuation char", badDataMalformed3),
    { 0, 0, 0 }
};



int main(int argc, char* argv[])
{
    if(argc == 2 && !strcmp(argv[1], "--print-summary")) {
        std::wcout << L"Regression test for the UTF-8 decoder.\n";
        return 0;
    }

    if(argc != 1) {
        std::wcout << L"No arguments expected.\n";
        return 1;
    }

    int ret = 0;
    try {
        MySimpleParser p;

        std::wcout << L"Checking OK data: " << std::flush;
        p.feedData(okData, sizeof(okData) - 1);
        std::wcout << L"success" << std::endl;

        for(TestData* test = testData; test->desc; ++test) {
            std::wcout << L"Checking bad data (" << test->desc
                    << L"): " << std::flush;
            p.feedBadData(test->str, test->len);
            std::wcout << "success" << std::endl;
            p.feedData(restartMarker, sizeof(restartMarker) - 1);
        }
    }
    catch(TestFailed&) {
        std::wcerr << L"Test failed.\n";
        ret = 1;
    }

    return ret;
}
